"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.PriorityQueueObserver = exports.observerErrors = void 0;
const errors_1 = require("../../../common/errors");
const stream_1 = require("stream");
const defekt_1 = require("defekt");
const observerErrors = defekt_1.defekt({
    ObserverError: {}
});
exports.observerErrors = observerErrors;
class PriorityQueueObserver {
    constructor(queue) {
        this.queue = queue;
        this.events = new stream_1.PassThrough({ objectMode: true });
    }
    emitIssue(issue) {
        this.events.push(issue);
    }
    async enqueue({ item, discriminator, priority }) {
        this.emitIssue({
            type: 'action',
            message: 'enqueue',
            data: item
        });
        try {
            await this.queue.enqueue({ item, discriminator, priority });
        }
        catch (ex) {
            this.emitIssue({
                type: 'error',
                message: 'An error occured during enqueue.',
                data: ex
            });
            throw ex;
        }
    }
    async lockNext() {
        try {
            this.emitIssue({
                type: 'action',
                message: 'lockNext'
            });
            const nextItem = await this.queue.lockNext();
            if (nextItem === undefined) {
                this.emitIssue({
                    type: 'issue',
                    message: 'Couldn\'t lock next item.'
                });
            }
            return nextItem;
        }
        catch (ex) {
            this.emitIssue({
                type: 'error',
                message: 'An error occured during lockNext.',
                data: ex
            });
            throw ex;
        }
    }
    async renewLock({ discriminator, token }) {
        try {
            this.emitIssue({
                type: 'action',
                message: 'renewLock',
                data: { discriminator, token }
            });
            await this.queue.renewLock({ discriminator, token });
        }
        catch (ex) {
            if (defekt_1.isCustomError(ex)) {
                switch (ex.code) {
                    case errors_1.errors.TokenMismatch.code:
                    case errors_1.errors.ItemNotFound.code:
                    case errors_1.errors.ItemNotLocked.code: {
                        this.emitIssue({
                            type: 'issue',
                            message: 'renewLock failed.',
                            data: ex
                        });
                        throw ex;
                    }
                    default: {
                        break;
                    }
                }
            }
            this.emitIssue({
                type: 'error',
                message: 'An error occured during renewLock.',
                data: ex
            });
            throw ex;
        }
    }
    async acknowledge({ discriminator, token }) {
        try {
            this.emitIssue({
                type: 'action',
                message: 'acknowledge',
                data: { discriminator, token }
            });
            await this.queue.acknowledge({ discriminator, token });
        }
        catch (ex) {
            if (defekt_1.isCustomError(ex)) {
                switch (ex.code) {
                    case errors_1.errors.TokenMismatch.code:
                    case errors_1.errors.ItemNotFound.code:
                    case errors_1.errors.ItemNotLocked.code: {
                        this.emitIssue({
                            type: 'issue',
                            message: 'acknowledge failed.',
                            data: ex
                        });
                        throw ex;
                    }
                    default: {
                        break;
                    }
                }
            }
            this.emitIssue({
                type: 'error',
                message: 'An error occured during acknowledge.',
                data: ex
            });
            throw ex;
        }
    }
    async defer({ discriminator, token, priority }) {
        try {
            this.emitIssue({
                type: 'action',
                message: 'defer',
                data: { discriminator, token, priority }
            });
            await this.queue.defer({ discriminator, token, priority });
        }
        catch (ex) {
            if (defekt_1.isCustomError(ex)) {
                switch (ex.code) {
                    case errors_1.errors.TokenMismatch.code:
                    case errors_1.errors.ItemNotFound.code:
                    case errors_1.errors.ItemNotLocked.code: {
                        this.emitIssue({
                            type: 'issue',
                            message: 'defer failed.',
                            data: ex
                        });
                        throw ex;
                    }
                    default: {
                        break;
                    }
                }
            }
            this.emitIssue({
                type: 'error',
                message: 'An error occured during defer.',
                data: ex
            });
            throw ex;
        }
    }
    async remove({ discriminator, itemIdentifier }) {
        try {
            this.emitIssue({
                type: 'action',
                message: 'remove',
                data: { discriminator, itemIdentifier }
            });
            await this.queue.remove({ discriminator, itemIdentifier });
        }
        catch (ex) {
            if (defekt_1.isCustomError(ex)) {
                switch (ex.code) {
                    case errors_1.errors.ItemNotFound.code: {
                        this.emitIssue({
                            type: 'issue',
                            message: 'remove failed.',
                            data: ex
                        });
                        throw ex;
                    }
                    default: {
                        break;
                    }
                }
            }
            this.emitIssue({
                type: 'error',
                message: 'An error occured during remove.',
                data: ex
            });
            throw ex;
        }
    }
    async setup() {
        return this.queue.setup();
    }
    // eslint-disable-next-line @typescript-eslint/no-shadow
    static async create({ observedQueue }) {
        return new PriorityQueueObserver(observedQueue);
    }
    async destroy() {
        this.events.destroy();
        return this.queue.destroy();
    }
    getEvents() {
        return this.events;
    }
}
exports.PriorityQueueObserver = PriorityQueueObserver;
PriorityQueueObserver.defaultExpirationTime = 15000;
//# sourceMappingURL=PriorityQueueObserver.js.map